/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2005  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM
#ifdef USE_NETWORK

#include "network.h"

#include "ui.h"
#include "cards.h"
#include "translations.h"

#include "../../network/server.h"
#include "../../network/connection.h"

#include "widgets/stock_button.h"
#include "widgets/labelspinbutton.h"
#include "widgets/labelentry.h"
#include <gtkmm/stock.h>
#include <gtkmm/treeview.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/alignment.h>
#include <gtkmm/frame.h>
#include <gtkmm/notebook.h>
namespace UI_GTKMM_NS {

  /**
   ** Constructor for the model
   **
   ** @param	-
   **
   ** @return	-
   ** 
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Network::ConnectionModel::ConnectionModel() :
    Gtk::TreeModel::ColumnRecord()
  {
    this->add(this->name);
    this->add(this->address);
    this->add(this->port);
#ifdef POSTPHONED
    this->add(this->found);
    this->add(this->color);
    this->add(this->accept);
#endif

    return ;
  } // Network::ConnectionModel::ConnectionModel()

  /**
   ** Constructor for the model
   **
   ** @param	-
   **
   ** @return	-
   ** 
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Network::LocalAddressesModel::LocalAddressesModel() :
    Gtk::TreeModel::ColumnRecord()
  {
    this->add(this->name);

    return ;
  } // Network::LocalAddressesModel::LocalAddressesModel()

  /**
   ** Constructor
   **
   ** @param	-
   **
   ** @return	-
   ** 
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Network::Network(Base* const parent) :
    Base(parent),
    StickyDialog("Network"),
    port(NULL),
    standard_port(NULL),
    server_running(NULL),
    connection_model(NULL),
    connection_list(),
    connection_treeview(NULL),
    new_connection_container(NULL),
    new_connection_address(NULL),
    new_connection_port(NULL),
    connect(NULL),
    local_addresses_model(NULL),
    local_addresses_list(),
    local_addresses_treeview(NULL)
  {
    this->init();
    this->signal_realize().connect(SigC::slot(*this, &Network::recreate_connections_list));
    this->signal_realize().connect(SigC::slot(*this, &Network::recreate_local_addresses_list));
    this->signal_realize().connect(SigC::slot(*this, &Network::update_sensitivity));

    return ;
  } // Network::Network(Base* const parent)

  /**
   ** Destruktor
   **
   ** @param	-
   **
   ** @return	-
   ** 
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Network::~Network()
  {
    return ;
  } // Network::~Network()

  /**
   ** create all subelements
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::init()
    {
      this->set_icon(this->ui->icon);
      this->ui->translations->add(*this, ::translator("Network settings"));

      this->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);

      { // action area
	Gtk::StockButton* close_button
	  = Gtk::manage(new Gtk::StockButton(Gtk::Stock::CLOSE,
					     "close"));
	this->ui->translations->add(*close_button->label,
				    ::translator("close"));
	this->add_action_widget(*close_button, Gtk::RESPONSE_CLOSE);
	close_button->signal_clicked().connect(SigC::slot(*this, &Gtk::Widget::hide));
      } // action area

      { // the elements
	this->port
	  = Gtk::manage(new Gtk::LabelSpinButton("port"));
	this->port->set_range(1024, 65535);
	this->port->set_value(::Network::Server::STANDARD_PORT);
	this->ui->translations->add(this->port->get_label(),
				    ::translator("port") + ":");

	this->standard_port
	  = Gtk::manage(new Gtk::StockButton("standard port"));
	this->ui->translations->add(*this->standard_port->label,
				    ::translator("standard port"));

	this->server_running
	  = Gtk::manage(new Gtk::CheckButton("server running"));
	this->ui->translations->add(*this->server_running,
				    ::translator("server running"));



	this->new_connection_address
	  = Gtk::manage(new Gtk::LabelEntry("address"));
	this->ui->translations->add(this->new_connection_address->get_label(),
				    ::translator("address") + ":");

	this->new_connection_port
	  = Gtk::manage(new Gtk::LabelSpinButton("port"));
	this->new_connection_port->set_range(1024, 65535);
	this->new_connection_port->set_value(::Network::Server::STANDARD_PORT);
	this->ui->translations->add(this->new_connection_port->get_label(),
				    ::translator("port") + ":");

	this->connect
	  = Gtk::manage(new Gtk::StockButton("connect"));
	this->ui->translations->add(*this->connect->label,
				    ::translator("connect"));
      } // the elements

      { // the connection table
	this->connection_model = new ConnectionModel();

	this->connection_treeview
	  = Gtk::manage(new Gtk::TreeView(Gtk::ListStore::create(*this->connection_model)));

	this->connection_list = Gtk::ListStore::create(*this->connection_model);
	this->connection_treeview = Gtk::manage(new Gtk::TreeView(this->connection_list));
	this->connection_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);

	this->connection_treeview->append_column("name", this->connection_model->name);
	this->connection_treeview->append_column("address", this->connection_model->address);
	this->connection_treeview->append_column("port", this->connection_model->port);
      } // the connection table
      { // the local addresses table
	this->local_addresses_model = new LocalAddressesModel();

	this->local_addresses_treeview
	  = Gtk::manage(new Gtk::TreeView(Gtk::ListStore::create(*this->local_addresses_model)));

	this->local_addresses_list
	  = Gtk::ListStore::create(*this->local_addresses_model);
	this->local_addresses_treeview
	  = Gtk::manage(new Gtk::TreeView(this->local_addresses_list));

	this->local_addresses_treeview->append_column("address", this->local_addresses_model->name);

	this->local_addresses_treeview->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
      } // the local addresses table

      { // Layout
	Gtk::VBox* vbox = Gtk::manage(new Gtk::VBox(false, 2 ex));
	this->get_vbox()->add(*vbox);
	{ // server
	  Gtk::Frame* frame = Gtk::manage(new Gtk::Frame("server"));
	  this->ui->translations->add(*dynamic_cast<Gtk::Label*>(frame->get_label_widget()),
				      ::translator("server"));
	  vbox->pack_start(*frame, false, false);

	  Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox(false, 1 em));
	  hbox->set_border_width(1 ex);
	  frame->add(*hbox);

	  hbox->add(*this->server_running);
	  hbox->add(*this->port);
	  hbox->add(*this->standard_port);
	} // server
	{ // create connection
	  this->new_connection_container = Gtk::manage(new Gtk::Frame);
	  vbox->pack_start(*this->new_connection_container, false, false);

	  Gtk::Label* label = Gtk::manage(new Gtk::Label("new connection"));
	  this->ui->translations->add(*label, ::translator("new connection"));
	  static_cast<Gtk::Frame*>(this->new_connection_container
				   )->set_label_widget(*label);

	  Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox(false, 1 em));
	  this->new_connection_container->add(*hbox);

	  hbox->set_border_width(1 ex);

	  hbox->add(*this->new_connection_address);
	  hbox->add(*this->new_connection_port);
	  hbox->add(*this->connect);
	} // create connection
	{ // notebook
	  Gtk::Notebook* notebook = Gtk::manage(new Gtk::Notebook);
	  vbox->pack_end(*notebook, true, true);
	  { // connections
	    Gtk::Label* label = Gtk::manage(new Gtk::Label("connections"));
	    this->ui->translations->add(*label, ::translator("connections"));

	    Gtk::ScrolledWindow* scrolled_window
	      = Gtk::manage(new Gtk::ScrolledWindow);
	    notebook->append_page(*scrolled_window, *label);

	    scrolled_window->set_border_width(1 ex);
	    scrolled_window->set_policy(Gtk::POLICY_NEVER,
					Gtk::POLICY_AUTOMATIC);
	    scrolled_window->add(*this->connection_treeview);
	    scrolled_window->set_hadjustment(this->connection_treeview->property_hadjustment());
	    scrolled_window->set_vadjustment(this->connection_treeview->property_vadjustment());
	  } // connections
#if 0
	  { // create connection
	    this->new_connection_container
	      = Gtk::manage(new Gtk::Alignment(Gtk::ALIGN_CENTER,
					       Gtk::ALIGN_CENTER,
					       0, 0));

	    Gtk::Label* label = Gtk::manage(new Gtk::Label("new connection"));
	    this->ui->translations->add(*label, ::translator("new connection"));
	    notebook->append_page(*this->new_connection_container, *label);

	    Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox(false, 1 em));
	    hbox->set_border_width(1 ex);
	    this->new_connection_container->add(*hbox);

	    hbox->add(*this->new_connection_address);
	    hbox->add(*this->new_connection_port);
	    hbox->add(*this->connect);
	  } // create connection
#endif
	  { // ip addresses
	    Gtk::Label* label = Gtk::manage(new Gtk::Label("local addresses"));
	    this->ui->translations->add(*label, ::translator("local addresses"));

	    Gtk::ScrolledWindow* scrolled_window
	      = Gtk::manage(new Gtk::ScrolledWindow);
	    notebook->append_page(*scrolled_window, *label);

	    scrolled_window->set_border_width(1 ex);
	    scrolled_window->set_policy(Gtk::POLICY_NEVER,
					Gtk::POLICY_AUTOMATIC);
	    scrolled_window->add(*this->local_addresses_treeview);
	    scrolled_window->set_hadjustment(this->local_addresses_treeview->property_hadjustment());
	    scrolled_window->set_vadjustment(this->local_addresses_treeview->property_vadjustment());
	  } // ip addresses
	} // notebook
      } // Layout

#ifdef DKNOF
      this->new_connection_address->get_entry().set_text("localhost");
#endif

      { // signals
	this->signal_show().connect(SigC::slot(*this, &Network::update_sensitivity));
	this->server_running->signal_toggled().connect(SigC::slot(*this, &Network::server_running_change_event));
	this->port->get_spin_button().signal_changed().connect(SigC::slot(*this, &Network::port_change_event));
	this->standard_port->signal_clicked().connect(SigC::slot(*this, &Network::reset_port_event));
	this->new_connection_address->get_entry().signal_changed().connect(SigC::slot(*this, &Network::update_sensitivity));
	this->new_connection_port->get_spin_button().signal_value_changed().connect(SigC::slot(*this, &Network::update_sensitivity));
	this->connect->signal_clicked().connect(SigC::slot(*this, &Network::connect_event));
      } // signals

      this->recreate_local_addresses_list();

      this->set_default_size(4 * this->ui->cards->width(),
			     3 * this->ui->cards->height());
      this->show_all_children();

      return ;
    } // void Network::init()

  /**
   ** updates the sensitivity
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::update_sensitivity()
    {
      bool const running = ::server->is_running();

      this->port->set_sensitive(!running);
      this->standard_port->set_sensitive(!running);

      this->connection_treeview->set_sensitive(running);
      this->new_connection_container->set_sensitive(running);

      { // check that the address is not already connected
	string const& address
	  = this->new_connection_address->get_text();
	if (address.empty())
	  this->connect->set_sensitive(false);
	else { // if !(address.empty())
	  unsigned const port
	    = static_cast<unsigned>(this->new_connection_port->get_value_as_int());
	  for (vector< ::Network::Connection*>::const_iterator
	       c = ::server->connections().begin();
	       c != ::server->connections().end();
	       ++c) {
	    if ((address == (*c)->address())
		&& (port == (*c)->port())) {
	      this->connect->set_sensitive(false);
	      break;
	    }
	  } // for (c \in ::server.connections())
	} // if !(address.empty())
      } // check that the address is not already connected

      return ;
    } // void Network::update_sensitivity()

  /**
   ** recreates the connections list
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::recreate_connections_list()
    {
      this->connection_list->clear();
      for (vector< ::Network::Connection*>::const_iterator
	   c = ::server->connections().begin();
	   c != ::server->connections().end();
	   ++c) {
	Gtk::TreeModel::Row row = *this->connection_list->append();
	row[this->connection_model->name] = (*c)->name();
	row[this->connection_model->address] = (*c)->address();
	row[this->connection_model->port] = (*c)->port();
      }

      return ;
    } // void Network::recreate_connections_list()

  /**
   ** recreates the local addresses list
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::recreate_local_addresses_list()
    {
      if (!::server)
	return ;

      vector<string> const local_addresses = ::server->local_addresses();
      this->local_addresses_list->clear();
      for (vector<string>::const_iterator
	   c = local_addresses.begin();
	   c != local_addresses.end();
	   ++c) {
	Gtk::TreeModel::Row row = *this->local_addresses_list->append();
	row[this->local_addresses_model->name] = *c;
      }

      return ;
    } // void Network::recreate_local_addresses_list()

  /**
   ** the server shall be started
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::server_running_change_event()
    {
      if (this->server_running->get_active())
	::server->start();
      else
	::server->stop();

      this->server_running->set_active(::server->is_running());

      return ;
    } // void Network::server_running_change_event()

  /**
   ** the port has been changed
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::port_change_event()
    {
      ::server->set_port(static_cast<unsigned>(this->port->get_value_as_int()));
      return ;
    } // void Network::port_change_event()

  /**
   ** the port is resettet to the standard port
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::reset_port_event()
    {
      this->port->set_value(::Network::Server::STANDARD_PORT);
      return ;
    } // void Network::reset_port_event()

  /**
   ** the server is started
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::server_started()
    {
      this->server_running->set_active(::server->is_running());
      this->recreate_local_addresses_list();
      this->update_sensitivity();
      return ;
    } // void UI_GTKMM::server_started() const

  /**
   ** the server is stopped
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::server_stopped()
    {
      this->server_running->set_active(::server->is_running());
      this->recreate_local_addresses_list();
      this->update_sensitivity();
      return ;
    } // void UI_GTKMM::server_stopped() const

  /**
   ** the connection 'connection' was opened
   **
   ** @param	connection	new connection
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::network_connection_new(::Network::Connection const& connection)
    {
      this->recreate_connections_list();
      return ;
    } // Network::network_connection_new(::Network::Connection connection)

  /**
   ** the connection 'connection' was changed
   **
   ** @param	connection	connection that was changed
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::network_connection_changed(::Network::Connection const& connection)
    {
      this->recreate_connections_list();
      return ;
    } // Network::network_connection_changed(::Network::Connection connection)

  /**
   ** the connection 'connection' was deleted
   **
   ** @param	connection	deleted connection
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::network_connection_deleted(::Network::Connection const& connection)
    {
      this->recreate_connections_list();
      return ;
    } // void Network::network_connection_deleted(::Network::Connection connection)


  /**
   ** the address of the new connection has changed
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::new_connection_address_change_event()
    {
      this->update_sensitivity();
      return ;
    } // void Network::new_connection_address_change_event()

  /**
   ** a connection shall be made
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::connect_event()
    {
      string const& address
	= this->new_connection_address->get_text();
      unsigned const port
	= static_cast<unsigned>(this->new_connection_port->get_value_as_int());
      ::server->create_connection(address, port);
      return ;
    } // void Network::connect_event()

  /**
   ** updates the language
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Network::language_update()
    {
      this->connection_treeview->get_column(0)->set_title(::translator("name").translation());
      this->connection_treeview->get_column(1)->set_title(::translator("address").translation());
      this->connection_treeview->get_column(2)->set_title(::translator("port").translation());

      this->local_addresses_treeview->get_column(0)->set_title(::translator("address").translation());

      return ;
    } // void Network::language_update()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_NETWORK
#endif // #ifdef USE_UI_GTKMM
